!
!  * calculates projected density of states convoluted with
!    a Gaussian
!
!  * On entry: amplitudes of projecting Wannier functions
!              eigenvalues
!  
!  * Writes the result to a file
!

  subroutine plotWpdos(                                              &
    U,          &! amplitudes of the Wannier functions
    eo,         &! eigenenergies of the complete system
    fileroot,   &! Wannier90 seedname
    numWann,    &! second dimension of U - number of Wans to plot
    numKPoints, &! second dimension of eo and third one of U
    numBands,   &! first dimension of eo and U
    ef)          ! Fermi level to be reported in the file

  implicit none

  integer, parameter            :: dp = selected_real_kind(15,300)

! Arguments
  complex(kind=dp),  intent(in) :: U(numBands,numWann,numKPoints)
  real(kind=dp),     intent(in) :: eo(numBands,numKPoints)
  integer,           intent(in) :: numWann,numKPoints,numBands
  real(kind=dp),     intent(in) :: ef
  character(len=100),intent(in) :: fileRoot

! Main array: the pdos
  real(dp),        allocatable:: pdos(:,:)

! iterators
  integer                     :: iband, ik,iE,ierr,iWann

! floats
  real(kind=dp)               :: weight,omega,eValue

  integer, parameter          :: ioUnit = 7

! Number of output energies, interval, energy stride
  integer, parameter          :: numE = 300
  real(dp),parameter          :: E1 = -8.0, E2 = +4.0,          &
                                     deltaE = (E2-E1)/numE
! \sqrt{\pi}
  real(dp),parameter          :: sqrtPi = 1.7724538509055159_dp

! smearing lenght
  real(dp),parameter          :: smearing = 0.2d0

! cut-off for the gaussian distribution
  real(dp),parameter          :: gaussTol = 1.0d-8  ! eV^-1
  real(dp)                    :: gaussRange         ! eV

! beginning/end index of sampling of the gaussian
  integer                     :: start,cease

! Smearing function:
  real(dp)                    :: smear,x,s
  smear(x,s) = exp(-x**2/s**2)/sqrtPi/s

! energy range for sampling of the Gaussian: gaussian = gaussTol
  gaussRange = smearing*sqrt(-1.0_dp*log(sqrtPi*smearing*gaussTol))

  allocate(pdos(numE,numWann))
  pdos = 0d0

! Smear eigenvalues and project...
  do iWann = 1,numWann
    do ik=1,numKPoints
    do iband=1,numBands
      weight   = abs(U(iband,iWann,ik))**2

! We sample the Gaussian in a restricted interval <start,cease>
        eValue = eo(iband,ik)
        start = (eValue - gaussRange - E1 - ef)/deltaE
        cease = (eValue + gaussRange - E1 - ef)/deltaE
        if (start.lt.1)      start = 1
        if (cease.gt.numE)   cease = numE

      do iE = start,cease
        omega = E1 + deltaE*iE +ef
        pdos(iE,iWann) = pdos(iE,iWann) + smear(omega-eValue,smearing)*weight
      enddo
      enddo
    enddo
  enddo

  pdos = pdos/dfloat(numKPoints)

!
! Write a file
!
! >> wdos <<
!
! Header factory
  open(unit=ioUnit,file=trim(fileRoot)//'.wdos',form='formatted',    &
       status='replace')  
  write(ioUnit,fmt=201)                                              &
       "Wannier function projected density of states"
  write(ioUnit,fmt=201)                                              &
       'System name '//                      trim(fileroot)
  write(ioUnit,fmt=202)                                              &
       "Gaussian smearing energy: "         ,smearing,"eV"
  write(ioUnit,fmt=200)                                              &
       'Number of bands '                   ,numBands
  write(ioUnit,fmt=200)                                              &
       'Number of k-points '                ,numKPoints
  write(ioUnit,fmt=200)                                              &
       'Number of Wannier functions '       ,numWann
  write(ioUnit,fmt=200)                                              &
       'Number of energy steps for plotting',numE
  write(ioUnit,fmt="('# ',a,':',f7.3,',',f7.3,' eV')")               &
       'Energy window for plotting '        ,E1,E2
  write(ioUnit,fmt=201)                                              &
       "Zero energy coresponds to the Fermi level"
  write(ioUnit,fmt=202)                                              &
       "originally                         ",ef," eV"
200 format ('# ',a,':',i5)
201 format ('# ',a)
202 format ('# ',a,f7.4,a)
!
! Iterate through available Wannier functions, calculate and write
!
  do iE = 1,numE
    omega = E1 + deltaE*iE
    write (unit=ioUnit,advance='no',fmt="(f9.4)") omega
    do iWann =1,numWann
      write (ioUnit,fmt="(f9.4)",advance='no') pdos(iE,iWann)
    enddo
    write (ioUnit,fmt="(f9.4)",advance='yes')
  enddo

  close(ioUnit)

  deallocate(pdos)

end subroutine plotWpdos
