Program WannierProjectedDensityOfStates
!
! Calculates the density of states projected on Wannier functions:
! pdos_m(\omega) = 
!    1/numKPoints \sum_{i=1}^{numKPoints}\sum_{n=1}^numBands
!    |<w_m|u_{n\mathbf k}>|^2 \delta(\omega-\epsilon_n(\mathbf k)
!
! With the smearing function
! \delta(\omega) = \frac{1}{s\sqrt\pi}e^{-\frac{\omega^2}{s^2}}
! governed by the parameter "s" (smearing).
!
! USAGE: the only argument is the Wannier90 seedname
! BEHAVIOR: needs seedname.mmn, seedname.chk, seedname.EIG and seedname.eig
! The first file is used to read matrix dimensions, the latter
! contains the U matrices in a binary format. Fermi energy must be in 
! the first line of the Siesta .EIG file; eigenvalues are
! read from the Swan .eig file. 
! Writes seedname.wdos organized as a gnuplot multicolumn file,
! the first column being energy with respect to the Fermi level
! and the other columns contain wpdos to the Wannier functions in
! the Wannier90 order.
! The program should detect and strip Swan's spin extensions in
! filenames when opening the .EIG file.
!
! GNU General Public Licence
!
!    wpdos is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    any later version.
!    wpdos is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with wpdos.  If not, see <http://www.gnu.org/licenses/>.

!
! Richard Koryt\'ar, 2009
!
! To use with VASP: just create a seedname.EIG 
! file that contains the Fermi energy in eV
!
! This version should be run with wannier90-1.2
!
! N. Lorente, 11/11/2011

  implicit none

  integer, parameter               :: dp = selected_real_kind(15,300)
  integer,parameter                :: ioUnit = 7
! Iterators
  integer                          :: iband, ik,nkp,i,j,k,iE,ierr,    &
                                      numWann,numNeigh,numBands,      &
                                      iiband,iik,iWann
! Input data
  integer                          :: numKPoints
  character(len=33)                :: mmnHeader,chkHeader
  character(len=20)                :: checkpoint
  character(len=100)               :: fileRoot
  logical                          :: have_disentangled
  real(kind=dp)                    :: latVec(3,3), recLatVec(3,3)
  real(kind=dp)                    :: omega_invariant
  logical,          allocatable    :: lwindow(:,:)
  integer,          allocatable    :: ndimwin(:)
  complex(kind=dp), allocatable    :: u_matrix_opt(:,:,:),u_matrix(:,:,:)
  complex(kind=dp), allocatable    :: U(:,:,:)         ! Wannier projection
  real(kind=dp),    allocatable    :: eo(:,:),kPointsFrac(:,:)
  real(kind=dp)                    :: ef                      ! fermi level

  integer                          :: iUp,iDn
  character(len=30)                :: EIGFileName
! NLmodified
  integer                          :: num_bands_, num_exclude_bands
  integer, allocatable             :: exclude_bands(:)
  integer                          :: mp_grid(3)
! NLmodified

  interface
    subroutine plotWpdos(                                              &
       U,          &! amplitudes of the Wannier functions
       eo,         &! eigenenergies of the complete system
       fileroot,   &! Wannier90 seedname
       numWann,    &! second dimension of U - number of Wans to plot
       numKPoints, &! second dimension of eo
       numBands,   &! first dimension of eo and U
       ef)          ! Fermi level to be reported in the file

      integer, parameter            :: dp = selected_real_kind(15,300)
   
   ! Arguments
      complex(kind=dp),  intent(in) :: U(numBands,numWann,numKPoints)
      real(kind=dp),     intent(in) :: eo(numBands,numKPoints)
      integer,           intent(in) :: numWann,numKPoints,numBands
      real(kind=dp),     intent(in) :: ef
      character(len=100),intent(in) :: fileroot
    end subroutine

  end interface


  call getArg(1,fileRoot,ierr)
  if (ierr.le.4) stop

! Now drop up/dn extensions for the Siesta EIG file
  iUp = index(fileRoot,".up",.true.)
  iDn = index(fileRoot,".dn",.true.)
  if (iUp.gt.0) then
    EIGFileName = fileRoot(1:iUp-1)//'.EIG'
    print *,EIGFileName
  elseif (iDn.gt.0) then
    EIGFileName = fileRoot(1:iDn-1)//'.EIG'
  endif
!
! >> EIG <<
!
  write(*,*) 'Reading Fermi level from file ',EIGFileName
  open(unit=ioUnit, file=EIGFileName,status="old", err=118)
  read(ioUnit,*) ef
  write(*,*)     "E_F = ",ef," eV"
  close(ioUnit)
!
! >> mmn <<
!
  write(*,*) 'Reading dimensions from file ',trim(fileRoot),'.mmn'
  open(unit=ioUnit, file=trim(fileRoot)//'.mmn',          &
    status="old", err=119)
  read(ioUnit,*) mmnHeader
  write(*,*)     mmnHeader
  read(ioUnit,*) numBands,numKPoints
  write(*,*)     numBands,numKPoints
  close(ioUnit)
!
! >> eig <<
!
  write(*,*) 'Reading eigenvalues from file ',trim(fileRoot),'.eig'
  open(unit=ioUnit, file=trim(fileRoot)//'.eig',          &
    status="old", err=120)
  allocate(eo(numBands,numKPoints))
  do ik = 1,numKPoints
    do iband = 1,numBands
      read(unit=ioUnit,fmt=*) iiband,iik,eo(iband,ik)
    enddo
  enddo
  close(ioUnit)
!
! >> chk <<
!
  write(*,'(1x,3a)') 'Reading restart information from file ',       &
    trim(fileRoot),'.chk :'
  open(unit=ioUnit,file=trim(fileRoot)//'.chk',status='old',            &
    form='unformatted',err=121)
  ! Read comment line
  read(ioUnit) chkHeader
  write(*,'(1x,a)',advance='no') "Checkpoint status: ",trim(chkHeader)

!NLmodified
  read(ioUnit) num_bands_
  read(ioUnit) num_exclude_bands
 allocate (exclude_bands(num_exclude_bands))
  read(ioUnit) (exclude_bands(i),i=1,num_exclude_bands)
!NLmodified

  read(ioUnit) ((latVec(i,j),i=1,3),j=1,3)     ! Real lattice
  read(ioUnit) ((recLatVec(i,j),i=1,3),j=1,3)  ! Reciprocal lattice
  read(ioUnit) numKPoints                      ! K-points

!NLmodified
  read(ioUnit) (mp_grid(i),i=1,3)                       ! M-P grid
!NLmodified

  allocate(kPointsFrac(3,numKPoints)) 
  read(ioUnit) ((kPointsFrac(i,nkp),i=1,3),nkp=1,numKPoints)
  read(ioUnit) numNeigh               ! nntot
  read(ioUnit) numWann                ! num_wann
  read(ioUnit) checkpoint             ! checkpoint
  checkpoint=adjustl(trim(checkpoint))
  write(*,*) checkpoint

  read(ioUnit) have_disentangled ! whether disentanglement has been performed
  if (have_disentangled) then
    read(ioUnit) omega_invariant      ! omega invariant
    ! lwindow
    allocate(lwindow(numBands,numKPoints),stat=ierr)
    read(ioUnit,err=122) ((lwindow(i,nkp),i=1,numBands),nkp=1,numKPoints)

    ! ndimwin
    allocate(ndimwin(numKPoints),stat=ierr)
    read(ioUnit,err=123) (ndimwin(nkp),nkp=1,numKPoints)

   ! U_matrix_opt
    allocate(u_matrix_opt(numBands,numWann,numKPoints),stat=ierr)
    read(ioUnit,err=124) (((u_matrix_opt(i,j,nkp),                   &
                        i=1,numBands),j=1,numWann),nkp=1,numKPoints)
  endif
  ! U_matrix
  allocate(u_matrix(numWann,numWann,numKPoints),stat=ierr)
  read(ioUnit,err=125) (((u_matrix(i,j,k),                           &
                        i=1,numWann),j=1,numWann),k=1,numKPoints)
  close(ioUnit)
!
! Input data inquire acomplished
!



! calculus
!
  allocate(U(numBands,numWann,numKPoints))
  if (have_disentangled) then
    forall (ik=1:numKPoints) &
      U(:,:,ik) = matmul(u_matrix_opt(:,:,ik),u_matrix(:,:,ik))
  else
    U = u_matrix
  endif

  deallocate(u_matrix)
  if (allocated(u_matrix_opt)) then
    deallocate(u_matrix_opt)
    deallocate(lwindow)
    deallocate(ndimwin)
  endif 

  call plotWpdos(U,eo,fileRoot,numWann,numKPoints,numBands,ef) 

  deallocate(U,eo)


  stop

118 write(*,*) 'Error opening '// EIGFileName
119 write(*,*) 'Error opening '//trim(fileRoot)//'.mmn'
120 write(*,*) 'Error opening '//trim(fileRoot)//'.eig'
121 write(*,*) 'Error opening '//trim(fileRoot)//'.chk'
122 write(*,*) 'Error reading lwindow from '//trim(fileRoot)//'.chk'
123 write(*,*) 'Error reading ndimwin from '//trim(fileRoot)//'.chk'
124 write(*,*) 'Error reading u_matrix_opt from '//trim(fileRoot)//'.chk'
125 write(*,*) 'Error reading u_matrix from '//trim(fileRoot)//'.chk'

!>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>X<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  
End Program WannierProjectedDensityOfStates
